# Pomeri lastnosti vezja

heads={
  'opus': {
    'simulator': 'SpiceOpus', 
	'settings': { 'debug': 0 }, 
	'moddefs': {
	  'def': { 'file': 'ojacevalnik.cir' }, 
	  'test': { 'file': 'testno.cir' }, 
	  'model': { 'file': 'cmos180n.lib', 'section': 'tm' }
	}
  }
}

analyses={
  'dc1': {
    'head': 'opus', 
	'modules': [ 'def', 'test' ], 
	'command': 'dc(-1.2, 1.2, "lin", 100, "vin", "dc")'
  }, 
  'ac1': {
    'head': 'opus', 
	'modules': [ 'def', 'test' ], 
	'command': 'ac(1, 1e9, "dec", 10)'
  }
}

corners={
  'nom': {
    'modules': [ 'model' ], 
	'params': {
	  'temperature': 25, 
	  'vdd': 1.8
	}
  }, 
  'tlo': {
    'modules': [ 'model' ], 
	'params': {
	  'temperature': -50, 
	  'vdd': 1.8
	}
  }, 
  'vddlo': {
    'modules': [ 'model' ], 
	'params': {
	  'temperature': 25, 
	  'vdd': 1.6
	}
  }
}

measures = {
  'swing': {
    'analysis': 'dc1', 
    'corners': [ 'nom', 'tlo', 'vddlo' ], 
    'expression': 'm.DCswingAtGain(v("out"), v("p", "n"), 0.5, "out")'
  },
  'gain': {
    'analysis': 'ac1', 
    'corners': [ 'nom', 'tlo', 'vddlo' ], 
    'expression': 'm.ACgain(m.ACtf(v("out"), v("p", "n")))'
  }, 
  'bw': {
    'analysis': 'ac1', 
    'corners': [ 'nom', 'tlo', 'vddlo' ], 
    'expression': 'm.ACbandwidth(m.ACtf(v("out"), v("p", "n")), scale())'
  }, 
  'pm': {
    'analysis': 'ac1', 
    'corners': [ 'nom', 'tlo', 'vddlo' ], 
    'expression': 'm.ACphaseMargin(m.ACtf(v("out"), v("p", "n")))'
  }, 
  'area': {
    'analysis': None, 
    'corners': [ 'nom' ], 
    'script': """
__result=0.0
__result+=param['wpmirr']*param['lpmirr']*(1+1)
__result+=param['wpout']*param['lpout']*16
__result+=param['wdif']*param['ldif']*(1+1)
__result+=param['wnmirr']*param['lnmirr']*(1+1+8)
__result+=param['rcomp']/1e3*12e-12
__result+=param['ccomp']/1e-12*100e-12
"""
  }
}

# TSMC .18
#         rsh    w              1k
# rnodw   4.1    0.22u .. 2u    53.7u x 0.22u = 11.8 um2
# 
# cmim    1u x 1u ... 1.025 fF  max 30u x 30u
#         1pF = 975 um2
# 
# cgate (pmos)  tox=4nm   eps0 = 8.85e-12    SiO2 eps = 3.9    Si3N4 eps = 7.5
# cgate (nmos)  tox=4nm
#               1u x 1u ...                  8.629 fF          16.59 fF
#                   1pF =                    115.9 um2         60.28 um2
#
# Izbrano: 1pF .. 100um2,  1k .. 12um2


from pyopus.evaluator.performance import PerformanceEvaluator

pe=PerformanceEvaluator(heads, analyses, corners, measures, debug=0)
pe({
	'wpmirr': 61.9e-6, 
	'lpmirr': 0.5e-6, 
	'wpout': 61.9e-6, 
	'lpout': 0.5e-6,
	'wdif': 90e-6, 
	'ldif': 0.5e-6,
	'wnmirr': 90e-6, 
	'lnmirr': 1.9e-6, 
	'rcomp': 1e3, 
	'ccomp': 1e-12
})
print(pe.formatResults())

pe.finalize()
