# Pomeri lastnosti vezja

heads={
  'opus': {
    'simulator': 'SpiceOpus', 
	'settings': { 'debug': 0 }, 
	'moddefs': {
	  'def': { 'file': 'ojacevalnik.cir' }, 
	  'test': { 'file': 'testno.cir' }, 
	  'model': { 'file': 'cmos180n.lib', 'section': 'tm' }
	}
  }
}

analyses={
  'dc1': {
    'head': 'opus', 
	'modules': [ 'def', 'test' ], 
	'command': 'dc(-1.2, 1.2, "lin", 100, "vin", "dc")'
  }, 
  'ac1': {
    'head': 'opus', 
	'modules': [ 'def', 'test' ], 
	'command': 'ac(1, 1e9, "dec", 10)'
  }
}

corners={
  'nom': {
    'modules': [ 'model' ], 
	'params': {
	  'temperature': 25, 
	  'vdd': 1.8
	}
  }, 
  'tlo': {
    'modules': [ 'model' ], 
	'params': {
	  'temperature': -50, 
	  'vdd': 1.8
	}
  }, 
  'vddlo': {
    'modules': [ 'model' ], 
	'params': {
	  'temperature': 25, 
	  'vdd': 1.6
	}
  }
}

measures = {
  'swing': {
    'analysis': 'dc1', 
    'corners': [ 'nom', 'tlo', 'vddlo' ], 
    'expression': 'm.DCswingAtGain(v("out"), v("p", "n"), 0.5, "out")'
  },
  'gain': {
    'analysis': 'ac1', 
    'corners': [ 'nom', 'tlo', 'vddlo' ], 
    'expression': 'm.ACgain(m.ACtf(v("out"), v("p", "n")))'
  }, 
  'bw': {
    'analysis': 'ac1', 
    'corners': [ 'nom', 'tlo', 'vddlo' ], 
    'expression': 'm.ACbandwidth(m.ACtf(v("out"), v("p", "n")), scale())'
  }, 
  'pm': {
    'analysis': 'ac1', 
    'corners': [ 'nom', 'tlo', 'vddlo' ], 
    'expression': 'm.ACphaseMargin(m.ACtf(v("out"), v("p", "n")))'
  }, 
  'area': {
    'analysis': None, 
    'corners': [ 'nom' ], 
    'script': """
__result=0.0
__result+=param['wpmirr']*param['lpmirr']*(1+1)
__result+=param['wpout']*param['lpout']*16
__result+=param['wdif']*param['ldif']*(1+1)
__result+=param['wnmirr']*param['lnmirr']*(1+1+8)
__result+=param['rcomp']/1e3*12e-12
__result+=param['ccomp']/1e-12*100e-12
"""
  }
}

mos_w = {
  'init': 50e-6, 
  'lo': 1e-6, 
  'hi': 95e-6, 
  'step': 0.01e-6
}

mos_l = {
  'init': 1e-6, 
  'lo': 0.18e-6, 
  'hi': 4e-6, 
  'step': 0.01e-6
}

costInput = { 
  'wpmirr': mos_w, 
  'lpmirr': mos_l, 
  'wpout': mos_w, 
  'lpout': mos_l, 
  'wdif': mos_w, 
  'ldif': mos_l, 
  'wnmirr': mos_w, 
  'lnmirr': mos_l, 
  'rcomp': {
    'init': 1e3, 
    'lo': 0.1e3, 
    'hi': 20e3, 
    'step': 0.01e3
  }, 
  'ccomp': {
    'init': 1e-12, 
    'lo': 0.001e-12, 
    'hi': 10e-12, 
    'step': 0.001e-12
  }
}

inputOrder = ['wpmirr', 'lpmirr', 'wpout', 'lpout', 'wdif', 'ldif', 'wnmirr', 'lnmirr', 'rcomp', 'ccomp']

costDefinition = [
  {
    'measure': 'swing', 
    'goal': 'MNabove(1.4)', 
  }, 
  {
    'measure': 'gain', 
    'goal': 'MNabove(73.0)', 
  }, 
  {
    'measure': 'bw', 
    'goal': 'MNabove(2.9e3)', 
  }, 
  {
    'measure': 'pm', 
    'goal': 'MNabove(75.0)', 
  }, 
  {
    'measure': 'area', 
    'goal': 'MNbelow(2e-9, 0.1e-9)', 
	'shape': 'CSlinear2(1.0, 0.001)', 
  }, 
]

# Dodaj vogale nazaj k meritvam
# Dobimo KF = 0.636 po 559 iteracij

from pyopus.evaluator.performance import PerformanceEvaluator
from pyopus.evaluator.cost import CostEvaluator, parameterSetup
from pyopus.optimizer import optimizerClass

pe=PerformanceEvaluator(heads, analyses, corners, measures, debug=0)
ce=CostEvaluator(pe, inputOrder, costDefinition, debug=0)

parametri=[
	61.9e-6, 
	0.5e-6, 
	61.9e-6, 
	0.5e-6,
	90e-6, 
	0.5e-6,
	90e-6, 
	1.9e-6, 
	1e3, 
	1e-12
]

(xini, xlo, xhi, xstep)=parameterSetup(inputOrder, costInput)
opt=optimizerClass("HookeJeeves")(ce, xlo=xlo, xhi=xhi, maxiter=1000)
opt.reset(xini)
opt.installPlugin(ce.getReporter())
opt.run()

xresult=opt.x

kf=ce(xresult)
print("Koncna kriterijska funkcija:%f" % kf)
print(pe.formatResults())
print(ce.formatParameters())
print(ce.formatResults())

pe.finalize()
